/*
 *   transmettre OSC vers un Arduino 
 */
 
#include  <stdio.h>
#include  <stdlib.h>
#include  <unistd.h>
#include  <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include  <termios.h>

#include  <getopt.h>
#include  <lo/lo.h>

#include  "serial.h"

/* ----------------------------------------------------------------- */

#define	LOCAL_PORT	"9000"
#define SERIAL_DEVICE   "/dev/ttyACM0"
#define DEFAULT_BAUDS	"9600"

int		verbosity = 0;
int		ttyfd  	  = -1;
/* ----------------------------------------------------------------- */

void error(int num, const char *msg, const char *path)
{
fprintf(stderr, "liblo server error %d in path %s : %s\n", num, path, msg);
exit(1);
}
/* ----------------------------------------------------------------- */
int button_handler(const char *path, const char *types, lo_arg ** argv,
                int argc, void *data, void *user_data)
{
int		button, value;
int		foo;

if (verbosity) fprintf(stderr, "%s : %s %s %d\n", __func__,
				path, types, argc);

if (2 != argc) {
	fprintf(stderr, "%s : bad argc %d\n", __func__, argc);
	return 1;
	}

button = argv[0]->i;
value  = argv[1]->i;

if (verbosity) fprintf(stderr, "%s : button %d is %d\n", __func__,
					button, value);

return 0;
}
/* ----------------------------------------------------------------- */
int xy_handler(const char *path, const char *types, lo_arg ** argv,
                int argc, void *data, void *user_data)
{

if (verbosity) fprintf(stderr, "%s : %s %s %d\n", __func__,
					path, types, argc);

return 0;
}
/* ----------------------------------------------------------------- */
/* ----------------------------------------------------------------- */
int read_all_from_serial(int fd)
{
char		byte;

while (1 == read(fd, &byte, 1)) {
	printf("%02x  %c\n", byte, byte);
	fflush(stdout);

	}
puts("");
return 0;
}
/* ----------------------------------------------------------------- */
static void help(int k)
{
puts("\t* osc2serial is a work in progress *");

puts("\t-b\tset serial baudrate (9600)");
puts("\t-p\tlistening UDP port ("LOCAL_PORT")");
puts("\t-s\tserial device ("SERIAL_DEVICE")");
puts("\t-v\tincrease verbosity");
exit(0);
}
/* ----------------------------------------------------------------- */

int main(int argc, char *argv[])
{
lo_server_thread st;
char		*local_port = LOCAL_PORT;
char		*serial_device = SERIAL_DEVICE;
int		opt;
int		Bauds;

Bauds = bauds_str_to_B( DEFAULT_BAUDS, 0);

/* parsing command line options */
while ((opt = getopt(argc, argv, "b:hp:s:v")) != -1) {
	switch (opt) {
		case 'b':	Bauds = bauds_str_to_B(optarg, 0);
								break;
		case 'h':	help(0);			break;
		case 'p':	local_port = optarg;		break;
		case 's':	serial_device = optarg;		break;
		case 'v':	verbosity++;			break;
		}
	}

/* checkin the parameters now */


ttyfd = open_serial(serial_device, Bauds, 0);
if (ttyfd < 0) {
	fprintf(stderr, "fail...\n");
	exit(1);
	}
	
read_all_from_serial(ttyfd);

exit(1);

st = lo_server_thread_new(local_port, error);

lo_server_thread_add_method(st, "/joystick/b", "ii", button_handler, NULL);
lo_server_thread_add_method(st, "/joystick/xy", "ii", xy_handler, NULL);

lo_server_thread_start(st);

for (;;) {
	fprintf(stderr, "t = %ld\n", time(NULL));
	sleep(1);
	}

return 0;
}
/* ----------------------------------------------------------------- */
