/*
 *		SHOW KONTROL
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <unistd.h>
#include  <limits.h>		/* for SHRT_MIN */
#include  <string.h>
#include  <time.h>
#include  <signal.h>
#include  <locale.h>

#include  <getopt.h>
#include  <lo/lo.h>

#include  "ncursefuncs.h"

/* ----------------------------------------------------------------- */

#define	LOCAL_PORT	"9001"

int		verbosity = 0;
unsigned long	hits = 0L;
int		must_redraw;

typedef struct {
	signed char		val;
	unsigned char		flags;
	} Avalue;

Avalue Values[128];

/* ----------------------------------------------------------------- */
int print_idx2ligcol(WINDOW *win, int idx, char *txt)
{
int	lig, col;

lig = (idx / 8) + 4;
col = (8*(idx % 8)) + 9;
mvwaddstr(win, lig, col, txt);
return 0;
}
/* ----------------------------------------------------------------- */
int redraw_values(int flag, int unused)
{
int		foo;
char		ligne[42];

for (foo=0; foo<128; foo++) {
	if (Values[foo].val < 0) {
		strcpy(ligne, "---");
		}
	else {	
		sprintf(ligne, "%3d", Values[foo].val);
		}
	if (Values[foo].flags & 1)	standout();
	print_idx2ligcol(stdscr, foo, ligne);
	if (Values[foo].flags & 1) {
		standend();
		Values[foo].flags &= ~1;
		}
	}
must_redraw = 0;

return -1;
}
/* ----------------------------------------------------------------- */
/* ----------------------------------------------------------------- */
int kontrol_handler(const char *path, const char *types, lo_arg ** argv,
                int argc, void *data, void *user_data)
{
int		knum, kval;

if (verbosity>1) fprintf(stderr, "%s : %s %s %d\n", __func__,
					path, types, argc);

knum = argv[0]->i;	kval = argv[1]->i;

if(verbosity)	fprintf(stderr, "%s : kontrol %3d %3d\n", __func__, knum, kval);

if (knum<0)	return -1;
if (knum>127)	return -2;

Values[knum].val = kval;
Values[knum].flags |= 1;
must_redraw = 1;

return 0;
}
/* ----------------------------------------------------------------- */
void error(int num, const char *msg, const char *path)
{
fprintf(stderr, "liblo server error %d in path %s : %s\n", num, path, msg);
exit(1);
}
/* ----------------------------------------------------------------- */
void init_machine(int unused)
{
int	foo;

for (foo=0; foo<128; foo++) {
	Values[foo].val = -1;		/* means 'invalid' */
	Values[foo].flags = 0;
	}
}
/* ----------------------------------------------------------------- */

int main(int argc, char *argv[])
{
lo_server_thread 	st;
char			*local_port = LOCAL_PORT;
int			opt;
char			ligne[81];
long			tdebut;

/* parsing command line options */
while ((opt = getopt(argc, argv, "p:vE:C:")) != -1) {
	switch (opt) {
		case 'p':	local_port = optarg;	break;
		case 'v':	verbosity++;		break;
		default:	exit(1);
		}
	}

init_machine(0);

tdebut = time(NULL);

st = lo_server_thread_new(local_port, error);

initcurses();
sprintf(ligne, ":%s ", local_port);
draw_main_screen(ligne, 0);
redraw_values(0, 0);

lo_server_thread_add_method(st, "/kontrol/v", "ii", kontrol_handler, NULL);

lo_server_thread_start(st);

for (;;) {
	if (verbosity) {
		sprintf(ligne, " %6ld s ", time(NULL)-tdebut);
		standout(); mvaddstr(0, 30, ligne); standend();
		}
	if (must_redraw) {
		redraw_values(0, 0);
		}
	usleep(50*1000);
	refresh();
	}

return 0;
}
/* ----------------------------------------------------------------- */
